/*
(c) Tim Dokchitser, redlib library, v3.0, October 2024, https://people.maths.bris.ac.uk/~matyd/redlib/

mmylib.m - standard basic functions and abbreviations
*/

/*
Usage: these are not intrinsics, and so have to be imported as follows

import "mmylib.m": Z, Q, PR, RFF, exp, Right, IsEvenZ, IsOddZ, writeq, writernq, 
  Count, IncludeAssoc, IncludeAssocIfNotIsomorphic, SortSet, trim, trimright, Last, DelSpaces, DelSymbols, DelCRs,
  ReplaceStringFunc, ReplaceString, VectorContent, SortBy, SortByFunc, SetAndMultiplicities, MultiSubsets,
  SetQAttribute, GetQAttribute, PrintSequence, HirzebruchJung, Dotted, Left, Swap, TeXPrintParameter,
  PolynomialFit, VertexChainToSequence, GraphLongestChain, PermutationsMultiset, writepiece, dxdyToAngle,
  AllPaths, PreferenceOrder, PreferenceOrder2, TeXPolynomial, Rewrite, TikzTransform, IsSubMultiset,
  TokenSplit, LabelledGraphFromMultigraph, PrintReal, IncidentVertices, SortedEndVertices, StringSplit;
*/

Z:=Integers();
Q:=Rationals();
PR:=PolynomialRing;
RFF:=RationalFunctionField;
exp:=func<sym,n|Sprintf(n ge 9 select "%o{%o}" else "%o%o",sym,n)>;  // sym = "_" or "^"
Left:=func<s,n|S[[1..Min(#S,n)]] where S is Sprint(s)>;
Right:=func<s,n|S[[Max(1,#S-n+1)..#S]] where S is Sprint(s)>;

IsEvenZ:=func<x|IsCoercible(Z,x/2)>;
IsOddZ:=func<x|IsCoercible(Z,(x-1)/2)>;

procedure Swap(~A,i,j)
  t:=A[i]; A[i]:=A[j]; A[j]:=t;
end procedure;

procedure Rewrite(filename: quiet:=false)   
  Flush(Open(filename,"w"));
end procedure;

procedure writeq(filename,str: rewrite:=false, newline:="\n")
  if rewrite then Rewrite(filename); end if;
  F:=Open(filename,"a");
  WriteBytes(F,[StringToCode(c): c in Eltseq(Sprint(str)*newline)]);
  Flush(F);
end procedure;

procedure writernq(filename,str)
  writeq(filename,str: rewrite:=true, newline:="");
end procedure;

function Count(S,x)                           // Count elements equal to x in S
  if Type(S) in [MonStgElt,ModTupRngElt] then S:=Eltseq(S); end if;
  return #[z: z in S | z eq x];
end function;

procedure IncludeAssoc(~A,i,d: init:={})
  if not assigned A then A:=AssociativeArray(); end if;
  if Type(init) in [SetEnum, SetIndx, SetMulti]
    then add:=Include;
    else add:=Append;
  end if;
  if IsDefined(A,i) 
    then add(~A[i],d); 
    else A[i]:=add(init,d); 
  end if;
end procedure;

procedure IncludeAssocIfNotIsomorphic(~A,i,d: init:={}, same:=IsIsomorphic)
  assert IsEmpty(init);
  if not assigned A then A:=AssociativeArray(); end if;
  if IsDefined(A,i) then 
    if exists{e: e in A[i] | same(d,e)} then return; end if;
    Include(~A[i],d); 
  else 
    A[i]:=Include(init,d); 
  end if;
end procedure;

function SortSet(X)           
  return Sort(SetToSequence(Set(X))); 
end function;

function trim(s) 
  n:=#s;
  while (n ge 2) and (s[n] in [" ","\n"]) do n-:=1; end while;
  f:=1;
  while (f le #s) and (s[f] in [" ","\n"]) do f+:=1; end while;
  if (f eq 1) and (n eq #s) then return s; end if;
  return s[[f..n]];
end function;

function trimright(s) 
  n:=#s;
  while (n ge 1) and (s[n] in [" ","\n"]) do n-:=1; end while;
  return s[[1..n]];
end function;

function Last(v)
  if Type(v) in [SeqEnum,List,SetIndx,Tup] then return v[#v]; end if;
  if HasSignature("Eltseq",[Type(v)]) then 
    w:=Eltseq(v); return w[#w]; 
  end if;
  error "Last: unrecognised type";
end function;

function DelSymbols(s,delsymb) 
  if Type(s) ne MonStgElt then s:=Sprint(s); end if;
  if Type(delsymb) eq MonStgElt then delsymb:=Eltseq(delsymb); end if;  
  s:=[c: c in Eltseq(s) | not (c in delsymb)];
  return IsEmpty(s) select "" else &cat s;
end function;

function DelSpaces(s)    // delete spaces from a string s
  return DelSymbols(s," \n");
end function;

DelCRs:=func<s|&cat([x: x in Eltseq(Sprint(s)) | x ne "\n"])>; // delete \n's

function ReplaceStringFunc(s,fs,ts)
  if Type(s) ne MonStgElt then s:=Sprint(s); end if;
  if Type(fs) eq SeqEnum then
    for i:=1 to #fs do
      s:=ReplaceStringFunc(s,fs[i],ts[i]);
    end for;
    return s;
  end if;
  return SubstituteString(s,fs,ts);
end function;

procedure ReplaceString(~s,fs,ts)
  s:=ReplaceStringFunc(s,fs,ts);
end procedure;

function VectorContent(v)
  den:=LCM([Denominator(x): x in v]);
  v:=[Z!(den*x): x in v];
  gcd:=GCD(v);
  if gcd eq 0 then return 0; end if;
  v:=[x div gcd: x in v];
  return gcd/den,v;
end function;

procedure SortBy(~L,f: sign:=1)
  if Type(L) eq SetEnum then
    L:=SetToSequence(L);
  end if;
  Sort(~L,func<a,b|fa lt fb select -sign else (fa eq fb select 0 else sign)
                   where fa is f(a) where fb is f(b)>);
end procedure;                

function SortByFunc(L,f: sign:=1)
  if Type(L) eq SetEnum then
    L:=SetToSequence(L);
  end if;
  return Sort(L,func<a,b|fa lt fb select -sign else (fa eq fb select 0 else sign)
                   where fa is f(a) where fb is f(b)>);
end function;                

procedure SortByPerm(~L,f,~perm: sign:=1)
  if Type(L) eq SetEnum then
    L:=SetToSequence(L);
  end if;
  Sort(~L,func<a,b|fa lt fb select -sign else (fa eq fb select 0 else sign)
                   where fa is f(a) where fb is f(b)>,~perm);
end procedure;                

function SetAndMultiplicities(X)
  S:=[x: x in Set(X)];
  M:=[Multiplicity(X,x): x in S];
  return S,M;
end function;

function GetQAttribute(attrname, init)
  Q:=Rationals();
  if attrname notin GetAttributes(FldRat) then
    AddAttribute(FldRat,attrname);
  end if;
  if not assigned Q``attrname then
    if Type(init) in [UserProgram,Intrinsic] then init:=init(); end if;
    Q``attrname:=init;
  end if;
  return Q``attrname;
end function;

procedure SetQAttribute(attrname, value)
  Q:=Rationals();
  if attrname notin GetAttributes(FldRat) then
    AddAttribute(FldRat,attrname);
  end if;
  Q``attrname:=value;
end procedure;

function PrintSequence(L: sep:=", ", mult:="^^%o", prfun:=Sprint, br:=false, d:=false, empty:="")    
  if IsEmpty(L) then return empty; end if;
  if Type(L) eq SetEnum then L:=SortSet(L); end if;
  if Type(L) eq SetMulti then L:=Sort([a: a in L]); end if;
  
  if (prfun cmpeq Sprint) and d then prfun:=DelSpaces; end if;
  if   (br cmpeq true) or br cmpeq "[" then obr:="["; cbr:="]"; 
  elif br cmpeq "("                    then obr:="("; cbr:=")"; 
  elif br cmpeq "{"                    then obr:="{"; cbr:="}"; 
                                       else obr:=""; cbr:="";              
  end if;
  if IsEmpty(L) then return obr*cbr; end if;
  if Type(L) eq SetMulti then 
    S,M:=SetAndMultiplicities(L);
    L:=[prfun(S[i])*(M[i] eq 1 select "" else Sprintf(mult,M[i])): i in [1..#S]];
    prfun:=Sprint;
  end if;
  return obr*&cat[prfun(L[i]) * (i eq #L select "" else sep): i in [1..#L]]*cbr;
end function;

function HirzebruchJung(r1,r2)                             
  // Shortest 1-path between rational numbers r1>=r2,
  // r1=n1/d1 > n2/d2 > ... > ni/di = r2 with all ni*d(i+1)-n(i+1)*di=1, e.g. 1/2 > 1/3 > 1/4
  // returns r1,r2 as part of the path (unless r1=r2, in which case returns [r1])
  // for various bits see Obus-Wewers arxiv 1805.09709v3
  error if r1 lt r2, "Expected r1>=r2";     // r1 >= r2 
  if r1 eq r2 then return [r1]; end if;
  I1:=Floor(r1);  
  I2:=Ceiling(r2); 
  if I1 gt I2 or (I1 eq I2 and #{r1,r2,I1} eq 3)  then     // integers in between 
    P1:=HirzebruchJung(r1,I1);
    if I1 eq I2 then P1:=Prune(P1); end if;
    PM:=[Q|n: n in [I1-1..I2+1 by -1]];
    P2:=HirzebruchJung(I2,r2);
    return P1 cat PM cat P2;
  end if;  
  I3:=Floor(r2);
  if I3 ne 0 then      // shift to [0,1]
    return [d+I3: d in HirzebruchJung(r1-I3,r2-I3)]; 
  end if;
  if r2 ne 0 then      // invert; decreases denominators and hence terminates
    return [1/d: d in Reverse(HirzebruchJung(1/r2,1/r1))];
  end if;
  L:=[r1];
  repeat               // go down to 0 with Euclid
    c:=Numerator(Last(L));
    d:=Denominator(Last(L));
    b:=InverseMod(c,d);
    a:=(b*c-1) div d;
    Append(~L,a/b);
  until a eq 0;
  return L;
end function;

function Dotted(s, n)
// Print s as sdfjdfkjsk...sdk for long strings
  s:=Type(s) eq MonStgElt select s else Sprint(s); 
  if #s le n then return s; end if;
  return s[[1..n-6]]*"..."*s[[#s-2..#s]];
end function;

function PolynomialFit(v,d)
// Recognises a polynomial function in n variables of total degree d by a set of values
// v=[<x1,...,xn>,f(x1,...,xn)]
// return false,-1 if degree too low, and false,0 if not enough values, f,1 if ok
  Q:=FieldOfFractions(Parent(v[1][2]));
  n:=#(v[1][1]);
  R<[z]>:=PR(Q,n);
  mons:=&join[MonomialsOfDegree(R,j): j in [0..d]];
  mat:=Matrix(Q,[[Evaluate(m,d[1]): m in mons]: d in v]);
  vec:=Vector(Q,[d[2]: d in v]);
  vprintf LSeries,1: "%o vectors %o monomials (deg %o)\n",#v,#mons,d;
  ok,sol,ker:=IsConsistent(Transpose(mat),vec);
  if not ok then return false,-1; end if;
  if Dimension(ker) ne 0 then return false,0; end if;
  return &+[Eltseq(sol)[i]*mons[i]: i in [1..#mons]],1;
end function;

function VertexChainToSequence(G,p)      // Convert a chain or a loop to sequence of vertices
  V:=Vertices(G);
  assert not IsEmpty(p);
  if #p eq 1 then return SortSet(EndVertices(p[1])); end if;
  e:=func<i|EndVertices(p[i])>;
  return [Representative(e(i) diff e(i+1)): i in [1..#p-1]] cat [Representative(e(#p) meet e(#p-1))] cat 
    [Representative(e(#p) diff e(#p-1))];
end function;

function GraphLongestChain(G)
  // Longest path as a sequence of edges
  V:=Vertices(G);
  maxp:=[];
  for i->v1 in V, j->v2 in V do 
    if i ge j then continue; end if;
    p:=ShortestPath(v1,v2);
    if #p le #maxp then continue; end if;
    maxp:=p;
  end for;
  p:=maxp;
  if IsEmpty(p) then return [V[1]]; end if;
  return VertexChainToSequence(G,p);  
end function;

function AllPaths(G,i,j: M:="uncomputed")   // All paths in a graph from G.i to G.j of shortest length (M=DistanceMatrix(G))
  if i eq j then return [[i]]; end if;
  if M cmpeq "uncomputed" then M:=DistanceMatrix(G); end if;
  V:=VertexSet(G);
  m:=M[i,j];
  v:=V.i;
  N:=[n: n in Neighbours(v) | M[Index(n),j] eq m-1];
  return [[i] cat p: p in AllPaths(G,Index(n),j: M:=M), n in N];
end function;

function PreferenceOrder(C,f)     // Sort C by the weight function f, and return order of every element,
  v:=[f(a): a in C];              // so that the minimal elements have order 1, next ones order 2, etc.
  s:=SortSet(v);
  return [Position(s,a): a in v];
end function;

function PreferenceOrder2(C,f)    // Sort 2-dim array C by the weight function f, and return order of every element,
  v:=[[f(a): a in c]: c in C];    // so that the minimal elements have order 1, next ones order 2, etc.
  s:=SortSet(&cat v);
  return [[Position(s,a): a in c]: c in v];
end function;


procedure TeXDollarAndExponents(~s)
  if Position(s,"$") ne 0 then 
    ReplaceString(~s,"$.1","\\alpha ");
    ReplaceString(~s,"$.2","\\beta ");
    ReplaceString(~s,"$.3","\\gamma ");
  end if;
  ReplaceString(~s,"^","!");
  repeat
    ok,b,c:=Regexp("[!]([0-9][0-9]+)",s);
    if not ok then break; end if;
    ReplaceString(~s,b,"^{"*c[1]*"}");
  until false;
  ReplaceString(~s,"!","^"); 
end procedure;

function TeXPolynomial(f: vnames:="default", vorder:="default", cfprint:=Sprint)
  
  if Type(f) in [FldFunRatUElt,FldFunRatMElt] then
    num:=TeXPolynomial(Numerator(f): vnames:=vnames, vorder:=vorder, cfprint:=cfprint);
    den:=TeXPolynomial(Denominator(f): vnames:=vnames, vorder:=vorder, cfprint:=cfprint);
    if den eq "1" then return num; end if;
    if exists{c: c in Eltseq(num) | c in ["*","+","/","-"]} then num:="("*num*")"; end if;
    if exists{c: c in Eltseq(den) | c in ["*","+","/","-"]} then num:="("*den*")"; end if;
    return Sprintf("%o/%o",num,den);
  end if;
          
  R<[v]>:=Parent(f);
  if vorder cmpeq "default" then 
    vorder:=[1..#v];
  end if;
  if vnames cmpeq "default" then 
    vnames:=[Sprint(R.i): i in [1..#v]];
  end if;
  s:="";
  x:=vorder[1];
  for i:=Degree(f,x) to 0 by -1 do
    c:=Coefficient(f,x,i);
    if c eq 0 then continue; end if;
    if #vorder eq 1 
      then cs:=cfprint(c);
      else cs:=TeXPolynomial(c: vnames:=vnames, vorder:=vorder[[2..#vorder]], cfprint:=cfprint);
    end if;
    if (Position(cs,"+") ne 0 or Position(cs[[2..#cs]],"-") ne 0) and (i gt 0) then
      cs:="("*cs*")";
    end if;
    if cs eq "1" then cs:=""; end if;
    if cs eq "-1" then cs:="-"; end if;
    if i ne 0 then cs*:=vnames[x]; end if;
    lbr:=i gt 9 or i lt 0 select "{" else "";
    rbr:=i gt 9 or i lt 0 select "}" else "";
    if i gt 1 then cs*:="^"*lbr*Sprint(i)*rbr; end if;
    if cs eq "" then cs:="1"; end if;
    if cs eq "-" then cs:="-1"; end if;
    if s eq "" then s:=cs; else s*:=(cs[1] eq "-" select "" else "+")*cs; end if;
  end for;
  TeXDollarAndExponents(~s);
  return s;
end function;


function TikzTransform(s,r,phi: prec:=2)
  // Scale by r and rotate by phi (degrees) a tikz picture in s
  // Affects (A) numeric positions (0.23,-0.12) and (B) anchor strings anchor=west

  error if Type(s) ne MonStgElt, "RotateTikz: Expected a tikz string in s";

  t:="";                               // accummulated output
  rad:=phi*Pi(RealField())/180;

  // Stage 1: Rotate numeric positions (x,y)
  repeat
    ok,_,B:=Regexp("(.*)[(](-?[.0-9]+),(-?[.0-9]+)[)](.*)",s);
    if not ok then t:=s*t; break; end if;
    x:=eval B[2];
    y:=eval B[3];
    xnew:=r*x*Cos(rad)-r*y*Sin(rad);
    ynew:=r*x*Sin(rad)+r*y*Cos(rad);
    t:=Sprintf("(%o,%o)%o%o",RealField(prec)!xnew,RealField(prec)!ynew,B[4],t);
    s:=B[1];
  until false;

  // Stage 2: rotate anchors anchor=north etc.
  s:=t;
  t:="";
  Directions:=[<"east",0>,<"north east",45>,<"north",90>,<"north west",135>,<"west",180>,
    <"south west",225>,<"south",270>,<"south east",315>];
  repeat
    ok,_,B:=Regexp("(.*)anchor[ ]*=[ ]*([a-zA-Z]+)(.*)",s);
    ok:=ok and (exists(a){d[2]: d in Directions | d[1] eq x} where x is ToLower(B[2]));
    if not ok then t:=s*t; break; end if;
    anew:=45*Round((a + phi)/45); 
    assert exists(d){d[1]: d in Directions | anew mod 360 eq d[2]};
    t:=Sprintf("anchor=%o%o%o",d,B[3],t);
    s:=B[1];
  until false;
  return t;
  
end function;


function TokenSplit(s,syms: trimtokens:=false)    // e.g. s="#max+b--c", syms="#a-z" -> [ "#max", "+", "b", "--", "c"]
  L:=[];
  s:=trim(s);
  repeat
    ok,_,b:=Regexp("(.*[^"*syms*"])(["*syms*"]+)(.*)",s);
    if not ok then break; end if;
    L cat:= [b[3],b[2]];
    s:=b[1];
  until false;
  ok,_,b:=Regexp("(["*syms*"]+)([^"*syms*"]+)",s);
  if ok  
    then L:=Reverse(L cat [b[2],b[1]]);
    else L:=Reverse(L cat [s]);
  end if;
  if trimtokens
    then return [s: t in L | #s ne 0 where s is trim(t)];
    else return [t: t in L | #t ne 0];
  end if;
end function;


function LabelledGraphFromMultigraph(V,E)
  // Given a multigraph G defined by 
  //   V = sequence of vertex labels
  //   E = sequence of [i,j,label] for edges {i,j} with labels
  // returns a labelled graph F which determines G up to isomorphism
  // Example:
  // C1:=LabelledGraphFromMultigraph([1,2,3],[[1,2,-3],[2,3,-1],[1,3,-2]]);
  // C2:=LabelledGraphFromMultigraph([2,3,1],[[1,2,-1],[2,3,-2],[1,3,-3]]);  // rotated version
  // IsIsomorphic(C1,C2);
  n:=#V;
  error if not IsEmpty(Set(V) meet {e[3]: e in E}), "Vertex and edge labels must be disjoint";
  F:=Graph<n|>;
  AssignLabels(VertexSet(F),V);
  for i->e in E do  
    v1,v2,elabel:=Explode(e);
    AddVertex(~F,elabel);
    AddEdge(~F,n+i,v1);
    AddEdge(~F,n+i,v2);
  end for;
  return F;
end function;


function PrintReal(x: prec:=28)
  if Abs(x) lt 10^-prec then return "0"; end if;
  x:=RealField()!x; 
  s:=Sprint(x);
  error if Position(s,"E") ne 0, "Floating point representation with exponent is not implemented";
  p:=Position(s,".");
  if p ne 0 then s:=Left(s,p+prec); end if;
  n:=#s;
  while (n gt 1) and (s[n] eq "0") do n-:=1; end while;
  if (n gt 1) and (s[n] eq ".") then n-:=1; end if;
  return s[[1..n]];
end function;


function IncidentVertices(v) 
  return {Representative(EndVertices(e) diff {v}): e in IncidentEdges(v)};
end function;


function SortedEndVertices(e: index:=false)
  v1,v2:=Explode(SetToSequence(EndVertices(e)));
  if Index(v1) le Index(v2) 
    then if index then return Index(v1),Index(v2); else return v1,v2; end if;
    else if index then return Index(v2),Index(v1); else return v2,v1; end if;
  end if;
end function;


procedure multiset_permutations(L, index, ~out)   // courtesy of ChatGPT 3.5
  if index eq #L then
    Append(~out,L); return;
  end if;
  used := {};     // Keep track of used elements to avoid repetitions
  for i in [index..#L] do
    if not L[i] in used then
      Swap(~L,index,i);      // Swap current element with i-th element
      multiset_permutations(L, index+1, ~out);  // Recur for next index
      Swap(~L,index,i);      // Undo the swap
      Include(~used,L[i]);   // Add L[i] to used
    end if;
  end for;
end procedure;


function PermutationsMultiset(S)
// List of all possible permutations of S, as a sequence of sequences, e.g. PermutationsMultiset({*1,1,1,2*}) -> 4 of them
  S:=[d: d in S];
  out:=[];
  multiset_permutations(S, 1, ~out);
  return out;
end function;


function IsSubMultiset(X,Y) 
// true if X is a subset of Y (both multisets)
  error if {Type(X),Type(Y)} ne {SetMulti}, "IsSubMultiset: X and Y should be multisets";
  return IsEmpty(X diff Y);
end function;


function StringSplit(s,P)
// Split a string into blocks [ [* type index, extracts *], ...] using an ordered sequence of patterns that describe types,
// e.g. P=[<mInt,"([0-9]+)">, <mEdgeSym,"([-=<>])">, ... ; used in ReductionType from string label
  L:=[];
  repeat
    found:=false;
    for d in P do
      ok,_,B:=Regexp("^"*d[2]*"(.*)$",s);
      if not ok then continue; end if;
      found:=true;
      Append(~L,[* d[1] *] cat [* B[i]: i in [1..#B-1] *]);
      s:=B[#B];
      break;    
    end for;
    error if not found, "StringSplit: could not match "*s;
  until #s eq 0;
  return L;
end function;


function TeXPrintParameter(s: tex:=true, sym:="", plainsym:="", texsym:="", printempty:=true)
  s:=Sprint(s);
  if #s eq 0 and not printempty then return ""; end if;
  sym:=sym eq "" select (tex select texsym else plainsym) else sym;
  par:=tex and #s gt 1 select "{"*s*"}" else s;
  return sym*par;
end function;


function MultiSubsets(multiset)    // Sequence of all multi-subsets of a given multiset 

  unique_elements := SetToSequence(Set(multiset));                          // unique elements 
  multiplicities := [Multiplicity(multiset,x): x in unique_elements];       // and their multiplicities
  out := [{**}];                           // Initialize the result with the empty multiset

  for i->elem in unique_elements do        // Iterate through each unique element and its multiplicity
    multiplicity := multiplicities[i];  
    current_subsets := [];                 // Generate all possible counts of the current element in the subsets
    for count in [0..multiplicity], S in out do
      current_subsets cat:= [S join {*elem^^count*}];     // Append the current count of the element to each S
    end for;
    out := current_subsets;                // Update the result with the new subsets including the current element
  end for;

  return out;
end function;


function writepiecestr(src,S,fromanchor,toanchor: filename:="")
  F:=Split(src:IncludeEmpty);
  fp:=Position(F,fromanchor);
  if fp eq 0 then error Sprintf("%o anchor not found%o",fromanchor,filename); end if;
  tp:=Position(F,toanchor,fp);
  if tp eq 0 then error Sprintf("%o anchor not found%o after %o",toanchor,filename,fromanchor); end if;
  F1:=PrintSequence(F[[1..fp]]: sep:="\n")*"\n";
  if Type(S) eq SeqEnum then S:=PrintSequence(S: sep:="\n"); end if;
  if Type(S) ne MonStgElt then S:=Sprint(S); end if;
  if #S ne 0 and Last(S) ne "\n" then S*:="\n"; end if;
  F3:=PrintSequence(F[[tp..#F]]: sep:="\n");
  return F1*S*F3*"\n";
end function;


procedure writepiece(filename,S,fromanchor,toanchor) 
  s:=writepiecestr(Read(filename),S,fromanchor,toanchor: filename:=" in "*filename);
  writernq(filename,s); 
end procedure;


function dxdyToAngle(dx,dy)
  if dx lt 0 then return 180+dxdyToAngle(-dx,-dy); end if;
  if Abs(dx) le Abs(dy)/10000 then return dy gt 0 select 90 else 270; end if;
  return Arctan(dy/dx)/Pi(RealField())*180;
end function;


